from uuid import uuid4

from django.contrib.auth.models import User
from django.db import models

class Artist(models.Model):
    scid = models.CharField(max_length=36, null=False, unique=True, verbose_name="Artist ID", default=str(uuid4()))
    is_mbid = models.BooleanField(verbose_name="MusicBrainz Artist ID", default=False)

    def get_related_names(self):
        return u' / '.join([unicode(art) for art in ArtistName.objects.filter(artist=self.pk)])
    get_related_names.short_description = 'Related names'
    name = property(get_related_names)

    def __unicode__(self):
        return self.name


class ArtistName(models.Model):
    name   = models.CharField(max_length=200, verbose_name="Artist name")
    artist = models.ForeignKey(Artist, null=False)

    def __unicode__(self):
        return self.name

class Album(models.Model):
    artist = models.ForeignKey(Artist)
    scid   = models.CharField(max_length=36, null=False, unique=True, verbose_name="Scrobble Album ID", default=str(uuid4()))
    is_mbid = models.BooleanField(verbose_name="MusicBrainz Album ID", default=False)

class AlbumName(models.Model):
    name   = models.CharField(max_length=200, unique=True, verbose_name="Album name")
    album  = models.ForeignKey(Album, null=False)

class Track(models.Model):
    artist = models.ForeignKey(Artist, null=False)
    album  = models.ForeignKey(Album, null=True)
    scid   = models.CharField(max_length=36, null=False, unique=True, verbose_name='Track ID', default=str(uuid4()))
    is_mbid = models.BooleanField(verbose_name='MusicBrainz Track ID', default=False)

    def __unicode__(self):
        art = u' / '.join([unicode(a) for a in ArtistName.objects.filter(artist=self.artist)])
        trk = u' / '.join([unicode(t) for t in TrackName.objects.filter(track=self)])  # This request should returns a single element, this call is less error-prone than an get.
        track = {'artist':art}
        track['title'] = trk
        return unicode('{artist} - {title}').format(**track)

    def get_name(self):
        return u' / '.join([unicode(t) for t in TrackName.objects.filter(track=self)])  # This request should returns a single element, this call is less error-prone than an get.
    get_name.short_description = 'Title / Track name'
    name = property(get_name)

class TrackName(models.Model):
    name   = models.CharField(max_length=200, verbose_name='track name')
    track  = models.ForeignKey(Track, null=False)

    def __unicode__(self):
        return unicode(self.name)

    def get_artist(self):
        return self.track.artist.name

class Scrobble(models.Model):
    user  = models.ForeignKey(User)
    track = models.ForeignKey(Track)
    date  = models.DateTimeField()

    def __unicode__(self):
        scrobble = unicode('{date} : {track}')
        return scrobble.format(date=self.date.strftime('%A, %d. %B %Y %I:%M%p'), track=self.track)

    def get_detailled_scrobble(self):
        return {'artist': self.track.artist.name, 'date':self.date.strftime('%s'), 'track':self.track.get_name()}
